import { scanUrl } from './scan/scanUrl';
import { scanBody } from './scan/scanBody';
import { scanHeaders } from './scan/scanHeaders';
import { scanFormData } from './scan/scanFormData';
import { scanCookies } from './scan/scanCookies';
import { ScanOptions } from './scan/interfaces/ITScanOptions';
import { ScanRequestParams } from './interfaces/ITScanRequestParams';

const defaultOptions: ScanOptions = {
  keys: true,
  values: true,
  detectSentences: true,
  detectReservedWords: true,
};

const defaultScanOptions = {
  checkUrl: defaultOptions,
  checkBody: defaultOptions,
  checkHeaders: defaultOptions,
  checkFormData: defaultOptions,
  checkCookies: defaultOptions,
};

export function scanRequest(params: ScanRequestParams): string[] {
  const {
    url = '',
    body = {},
    headers = {},
    formData = new FormData(),
    cookies = {},
    options = defaultScanOptions,
  } = params;

  const mergedOptions = {
    checkUrl: { ...defaultOptions, ...options.checkUrl },
    checkBody: { ...defaultOptions, ...options.checkBody },
    checkHeaders: { ...defaultOptions, ...options.checkHeaders },
    checkFormData: { ...defaultOptions, ...options.checkFormData },
    checkCookies: { ...defaultOptions, ...options.checkCookies },
  };

  let issues: string[] = [];

  if (url !== '') {
    const checkUrl = mergedOptions.checkUrl;
    if (checkUrl.values) {
      issues = issues.concat(scanUrl(url, checkUrl));
    }
  }

  if (Object.keys(body).length > 0) {
    const checkBody = mergedOptions.checkBody;
    issues = issues.concat(scanBody(body, checkBody));
  }

  if (Object.keys(headers).length > 0) {
    const checkHeaders = mergedOptions.checkHeaders;
    issues = issues.concat(scanHeaders(headers, checkHeaders));
  }

  if (formData.entries().next().done === false) {
    const checkFormData = mergedOptions.checkFormData;
    issues = issues.concat(scanFormData(formData, checkFormData));
  }

  if (Object.keys(cookies).length > 0) {
    const checkCookies = mergedOptions.checkCookies;
    issues = issues.concat(scanCookies(cookies, checkCookies));
  }

  return issues;
}
